import axios, { AxiosError, AxiosResponse, AxiosRequestConfig } from "axios";

import { BASE_URL } from "../config";
import { logger } from "./util";
import { store } from "../../store";

const getToken = () => {
  const token = store.getState().auth.token;
  return token;
};

axios.interceptors.response.use(
  (response) => response,
  (error: AxiosError) => {
    return Promise.reject(error);
  }
);

export const request = (
  url: AxiosRequestConfig["url"],
  method: AxiosRequestConfig["method"],
  requestData?: AxiosRequestConfig["data"] | AxiosRequestConfig["params"],
  isGuest?: boolean,
  contentType?: string
) =>
  new Promise(async (resolve, reject) => {
    const endpoint = url?.replace?.("<BASE_URL>", BASE_URL);

    const params = method === "GET" ? requestData : null;
    const data = method === "GET" ? null : requestData;

    const auth_token = !isGuest ? await getToken() : null;
    const headers = {
      auth_token,
      "Content-Type": contentType || "application/json",
    };

    logger("REQUEST: ", method, endpoint, headers, requestData);

    axios({
      url: endpoint,
      method,
      data,
      params,
      headers,
      timeout: 30000,
    })
      .then(async (response: AxiosResponse) => {
        logger("RESPONSE: ", response);
        resolve(response.data);
      })
      .catch(async (error: AxiosError) => {
        if (error?.response) {
          logger("ERROR RESPONSE: ", error?.response);
          return reject(error?.response?.data);
        } else if (error?.request) {
          logger("NO RESPONSE: ", error?.request);
        } else {
          logger("SETUP ISSUE: ", error?.message);
        }
        reject(error);
      });
  });
