import { takeLeading, call, put, select } from "redux-saga/effects";
import { ACTIONS } from "../actions";
import CommonAPI from "../apis/common";
import { APP_STATE, JobType, Reducers } from "../types";

function* getJobs({
  success,
}: {
  type: typeof ACTIONS.GET_JOBS;
  success?: () => void;
}) {
  try {
    const data: { jobs: JobType; success: boolean } = yield call(
      CommonAPI.getJobs
    );
    if (data.success) {
      yield put({ type: ACTIONS.SET_JOBS, payload: data.jobs });
    }

    success?.();
  } catch (error) {
    success?.();
  }
}

function* createJob({
  payload,
  success,
}: {
  type: typeof ACTIONS.CREATE_JOB;
  payload: JobType;
  success: () => void;
}) {
  try {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });
    const data: { success: boolean; job: JobType } = yield call(
      CommonAPI.createJob,
      payload
    );

    if (data.success) {
      const jobs: JobType[] = yield select(
        (state: Reducers) => state.common.jobs
      );

      yield put({ type: ACTIONS.SET_JOBS, payload: [data.job, ...jobs] });
    }

    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.SUCCESS },
    });
    success();
  } catch (error) {
    success();
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.FAILED },
    });
  }
}

function* updateJob({
  payload,
  success,
}: {
  type: typeof ACTIONS.UPDATE_JOB;
  payload: JobType;
  success: () => void;
}) {
  try {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });

    const data: { success: boolean } = yield call(CommonAPI.updateJob, payload);

    if (data.success) {
      let jobs: JobType[] = yield select(
        (state: Reducers) => state.common.jobs
      );
      const found = jobs.findIndex((_job) => _job._id === payload._id);
      jobs[found] = { ...jobs[found], ...payload };

      yield put({ type: ACTIONS.SET_JOBS, payload: [...jobs] });
    }

    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.SUCCESS },
    });
    success();
  } catch (error) {
    success();
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.FAILED },
    });
  }
}

function* updateApplication({
  payload,
  success,
}: {
  type: typeof ACTIONS.UPDATE_APPLICATION;
  payload: { applicationId: string; update: any; candidateId: string };
  success?: () => void;
}) {
  try {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });

    const data: { success: boolean } = yield call(
      CommonAPI.updateApplication,
      payload
    );

    if (data.success) {
      yield call(getJobs, { type: ACTIONS.GET_JOBS });
    }

    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.SUCCESS },
    });
    success?.();
  } catch (error) {
    success?.();
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.FAILED },
    });
  }
}

export default function* authSaga() {
  yield takeLeading(ACTIONS.GET_JOBS, getJobs);
  yield takeLeading(ACTIONS.CREATE_JOB, createJob);
  yield takeLeading(ACTIONS.UPDATE_JOB, updateJob);
  yield takeLeading(ACTIONS.UPDATE_APPLICATION, updateApplication);
}
